@@ -62,7 +62,7 @@ class SST1RSoXSLoader(FileLoader):
 
 
 
-    def loadSingleImage(self,filepath,coords=None, return_q=False):
+    def loadSingleImage(self,filepath,coords=None, return_q=False,**kwargs):
         '''
         HELPER FUNCTION that loads a single image and returns an xarray with either pix_x / pix_y dimensions (if return_q == False) or qx / qy (if return_q == True)
 
@@ -73,6 +73,8 @@ class SST1RSoXSLoader(FileLoader):
             return_q (bool): return qx / qy coords.  If false, returns pixel coords.
 
         '''
+        if len(kwargs.keys())>0:
+            warnings.warn(f'Loader does not support features for args: {kwargs.keys()}',stacklevel=2)
         
         img = Image.open(filepath)
 
